@@ -127,6 +127,33 @@ class BiasAdd(Layer):
         return K.bias_add(inputs, self.bias)
 
 
+class Concatenate1D(Layer):
+    """1维序列拼接层
+    说明：本来该功能可以直接通过Concatenate层来实现，无奈Keras
+          自带的Concatenate层的compute_mask写得不合理，导致一个
+          带mask的序列与一个不带mask的序列拼接会报错，因此干脆
+          自己重写一个好了。
+    """
+    def call(self, inputs):
+        return K.concatenate(inputs, axis=1)
+
+    def compute_mask(self, inputs, mask=None):
+        if mask is not None:
+            masks = []
+            for i, m in enumerate(mask):
+                if m is None:
+                    m = K.ones_like(inputs[i][..., 0], dtype='bool')
+                masks.append(m)
+            return K.concatenate(masks, axis=1)
+
+    def compute_output_shape(self, input_shape):
+        if all([shape[1] for shape in input_shape]):
+            seq_len = sum([shape[1] for shape in input_shape])
+            return (input_shape[0][0], seq_len, input_shape[0][2])
+        else:
+            return (input_shape[0][0], None, input_shape[0][2])
+
+
 class MultiHeadAttention(Layer):
     """多头注意力机制
     """
@@ -394,13 +421,14 @@ class LayerNormalization(Layer):
 
 
 class PositionEmbedding(Layer):
-    """定义位置Embedding，这里的Embedding是可训练的。
+    """定义可训练的位置Embedding
     """
     def __init__(
         self,
         input_dim,
         output_dim,
         merge_mode='add',
+        hierarchical=None,
         embeddings_initializer='zeros',
         custom_position_ids=False,
         **kwargs
@@ -409,6 +437,7 @@ class PositionEmbedding(Layer):
         self.input_dim = input_dim
         self.output_dim = output_dim
         self.merge_mode = merge_mode
+        self.hierarchical = hierarchical
         self.embeddings_initializer = initializers.get(embeddings_initializer)
         self.custom_position_ids = custom_position_ids
 
@@ -423,29 +452,43 @@ class PositionEmbedding(Layer):
     def call(self, inputs):
         """如果custom_position_ids，那么第二个输入为自定义的位置id
         """
+        input_shape = K.shape(inputs)
+        batch_size, seq_len = input_shape[0], input_shape[1]
+
         if self.custom_position_ids:
             inputs, position_ids = inputs
             if K.dtype(position_ids) != 'int32':
                 position_ids = K.cast(position_ids, 'int32')
-            pos_embeddings = K.gather(self.embeddings, position_ids)
         else:
-            input_shape = K.shape(inputs)
-            batch_size, seq_len = input_shape[0], input_shape[1]
-            pos_embeddings = self.embeddings[:seq_len]
-            pos_embeddings = K.expand_dims(pos_embeddings, 0)
-            if self.merge_mode != 'add':
-                pos_embeddings = K.tile(pos_embeddings, [batch_size, 1, 1])
+            position_ids = K.arange(0, seq_len, dtype='int32')[None]
+
+        if self.hierarchical:
+            alpha = 0.4 if self.hierarchical is True else self.hierarchical
+            embeddings = self.embeddings - alpha * self.embeddings[:1]
+            embeddings = embeddings / (1 - alpha)
+            embeddings_x = K.gather(embeddings, position_ids // self.input_dim)
+            embeddings_y = K.gather(embeddings, position_ids % self.input_dim)
+            pos_embeddings = alpha * embeddings_x + (1 - alpha) * embeddings_y
+        else:
+            if self.custom_position_ids:
+                pos_embeddings = K.gather(self.embeddings, position_ids)
+            else:
+                pos_embeddings = self.embeddings[None, :seq_len]
 
         if self.merge_mode == 'add':
             return inputs + pos_embeddings
+        elif self.merge_mode == 'mul':
+            return inputs * pos_embeddings
         else:
+            if not self.custom_position_ids:
+                pos_embeddings = K.tile(pos_embeddings, [batch_size, 1, 1])
             return K.concatenate([inputs, pos_embeddings])
 
     def compute_output_shape(self, input_shape):
         if self.custom_position_ids:
             input_shape = input_shape[0]
 
-        if self.merge_mode == 'add':
+        if self.merge_mode in ['add', 'mul']:
             return input_shape
         else:
             return input_shape[:2] + (input_shape[2] + self.output_dim,)
@@ -455,6 +498,7 @@ class PositionEmbedding(Layer):
             'input_dim': self.input_dim,
             'output_dim': self.output_dim,
             'merge_mode': self.merge_mode,
+            'hierarchical': self.hierarchical,
             'embeddings_initializer':
                 initializers.serialize(self.embeddings_initializer),
             'custom_position_ids': self.custom_position_ids,
@@ -463,6 +507,67 @@ class PositionEmbedding(Layer):
         return dict(list(base_config.items()) + list(config.items()))
 
 
+class SinusoidalPositionEmbedding(Layer):
+    """定义Sin-Cos位置Embedding
+    """
+    def __init__(
+        self, output_dim, merge_mode='add', custom_position_ids=False, **kwargs
+    ):
+        super(SinusoidalPositionEmbedding, self).__init__(**kwargs)
+        self.output_dim = output_dim
+        self.merge_mode = merge_mode
+        self.custom_position_ids = custom_position_ids
+
+    def call(self, inputs):
+        """如果custom_position_ids，那么第二个输入为自定义的位置id
+        """
+        input_shape = K.shape(inputs)
+        batch_size, seq_len = input_shape[0], input_shape[1]
+
+        if self.custom_position_ids:
+            inputs, position_ids = inputs
+        else:
+            position_ids = K.arange(0, seq_len, dtype=K.floatx())[None]
+
+        indices = K.arange(0, self.output_dim // 2, dtype=K.floatx())
+        indices = K.pow(10000.0, -2 * indices / self.output_dim)
+        pos_embeddings = tf.einsum('bn,d->bnd', position_ids, indices)
+        pos_embeddings = K.concatenate([
+            K.sin(pos_embeddings)[..., None],
+            K.cos(pos_embeddings)[..., None]
+        ])
+        pos_embeddings = K.reshape(
+            pos_embeddings, (-1, seq_len, self.output_dim)
+        )
+
+        if self.merge_mode == 'add':
+            return inputs + pos_embeddings
+        elif self.merge_mode == 'mul':
+            return inputs * pos_embeddings
+        else:
+            if not self.custom_position_ids:
+                pos_embeddings = K.tile(pos_embeddings, [batch_size, 1, 1])
+            return K.concatenate([inputs, pos_embeddings])
+
+    def compute_output_shape(self, input_shape):
+        if self.custom_position_ids:
+            input_shape = input_shape[0]
+
+        if self.merge_mode in ['add', 'mul']:
+            return input_shape
+        else:
+            return input_shape[:2] + (input_shape[2] + self.output_dim,)
+
+    def get_config(self):
+        config = {
+            'output_dim': self.output_dim,
+            'merge_mode': self.merge_mode,
+            'custom_position_ids': self.custom_position_ids,
+        }
+        base_config = super(SinusoidalPositionEmbedding, self).get_config()
+        return dict(list(base_config.items()) + list(config.items()))
+
+
 class RelativePositionEmbedding(Layer):
     """相对位置编码
     来自论文：https://arxiv.org/abs/1803.02155
@@ -1003,9 +1108,11 @@ class Loss(Layer):
 custom_objects = {
     'Embedding': Embedding,
     'BiasAdd': BiasAdd,
+    'Concatenate1D': Concatenate1D,
     'MultiHeadAttention': MultiHeadAttention,
     'LayerNormalization': LayerNormalization,
     'PositionEmbedding': PositionEmbedding,
+    'SinusoidalPositionEmbedding': SinusoidalPositionEmbedding,
     'RelativePositionEmbedding': RelativePositionEmbedding,
     'RelativePositionEmbeddingT5': RelativePositionEmbeddingT5,
     'FeedForward': FeedForward,
